STOCK_TICKER_NAMES = {
    'AAPL': 'Apple Inc.',
    'MSFT': 'Microsoft Corporation',
    'AMZN': 'Amazon.com, Inc.',
    'GOOG': 'Alphabet Inc. (Class C)',
    'TSLA': 'Tesla, Inc.',
    'BABA': 'Alibaba Group Holding Limited (ADR)',
    'NVDA': 'NVIDIA Corporation',
    'JPM': 'JPMorgan Chase & Co.',
    'V': 'Visa Inc.',
    'BAC': 'Bank of America Corporation',
    'WMT': 'Walmart Inc.',
    'MA': 'Mastercard Incorporated',
    'PG': 'The Procter & Gamble Company',
    'JNJ': 'Johnson & Johnson',
    'KO': 'The Coca-Cola Company',
    'PEP': 'PepsiCo, Inc.',
    'DIS': 'The Walt Disney Company',
    'LVMUY': 'LVMH Moët Hennessy - Louis Vuitton, Société Européenne (ADR)',
    'LMT': 'Lockheed Martin Corporation',
    'NKE': 'NIKE, Inc.',
    'HDB': 'HDFC Bank Limited (ADR)',
    'TM': 'Toyota Motor Corporation (ADR)',
    'PTR': 'PetroChina Company Limited (ADR)',
    'RIO': 'Rio Tinto Group',
    'BHP': 'BHP Group',
    'SNY': 'Sanofi',
    'NVO': 'Novo Nordisk A/S (ADR)',
    'EL': 'The Estée Lauder Companies Inc.',
    'LRLCY': 'L\'Oréal S.A. (ADR)',
    'SAN': 'Banco Santander, S.A. (ADR)',
    'ING': 'ING Groep N.V. (ADR)',
    'ICAGY': 'International Consolidated Airlines Group, S.A. (ADR)',
    'BTI': 'British American Tobacco p.l.c. (ADR)',
    'UL': 'Unilever PLC (ADR)',
    'BP': 'BP p.l.c.',
    'TOT': 'TotalEnergies SE',
    'VOD': 'Vodafone Group Plc (ADR)',
    'NESN': 'Nestlé S.A.',
    'UBER': 'Uber Technologies, Inc.',
    'DTEGY': 'Deutsche Telekom AG (ADR)',
    'BMWYY': 'Bayerische Motoren Werke Aktiengesellschaft (ADR)',
    'TCEHY': 'Tencent Holdings Limited (ADR)',
    'NOK': 'Nokia Corporation (ADR)',
    'AZN': 'AstraZeneca PLC (ADR)',
    'SNE': 'Sony Corporation (ADR)',
    '9984.T': 'SoftBank Group Corp.',
    '6502.T': 'Toshiba Corporation',
    '7203.T': 'Toyota Motor Corporation',
    'AAPL34.SA': 'Apple Inc.',
    'VALE3.SA': 'Vale S.A.',
    'B3SA3.SA': 'B3 S.A. - Brasil, Bolsa, Balcão',
    'PETR4.SA': 'Petroleo Brasileiro S.A. - Petrobras'
}
